import os
from datetime import datetime
from openpyxl import Workbook
from openpyxl.utils import get_column_letter
from openpyxl.styles import Font, PatternFill, Alignment
import requests
from bs4 import BeautifulSoup
import re

# スクリプトのあるフォルダを基準にディレクトリパスを指定
dir_path = os.path.dirname(os.path.abspath(__file__))

# 現在時刻からExcelファイル名を生成
now = datetime.now()
filename = f"同人作品ファイルリスト_{now.strftime('%Y%m%d_%H%M')}.xlsx"

# Excelファイルを作成
wb = Workbook()
ws = wb.active
ws.title = "Subfolders"
ws.append(["カテゴリー", "製作者", "作品名", "サブフォルダパス", "更新日時",  "タグ"])

# ヘッダーの書式設定
for col in ws.iter_cols(min_row=1, max_row=1):
    col[0].font = Font(bold=True)
    col[0].fill = PatternFill(start_color="ADD8E6", end_color="ADD8E6", fill_type="solid")
for row in ws.iter_rows(min_row=1, max_row=1):
    for cell in row:
        cell.alignment = Alignment(horizontal="center", vertical="center")

# サブフォルダの走査
for subdir in os.scandir(dir_path):
    if subdir.is_dir():
        subdir_path = subdir.path
        subdir_name = subdir.name
        if subdir_name.startswith("（") and "）[" in subdir_name and "]" in subdir_name:
            # カテゴリー、製作者、作品名を抽出
            category_start = subdir_name.find("（") + 1  # 最初の "（" の位置
            category_end = subdir_name.find("）")       # 最初の "）" の位置
            category = subdir_name[category_start:category_end]  # "（" と "）" の間の文字列を取得
            author_start = subdir_name.find("[") + 1  # 最初の "[" の位置
            author_end = subdir_name.find("]")       # 最初の "]" の位置
            author = subdir_name[author_start:author_end]  # "[" と "]" の間の文字列を取得
            title_start = subdir_name.find("] ") + 2  # 最初の "] " の位置 + 2
            title = subdir_name[title_start:]  # "作品名" の文字列を取得

            # DLSiteの検索結果ページからタイトルと一致する作品ページのURLと作品名を取得する
            keyword = title.replace("（オリジナル）","")
            keyword = keyword.replace(" ", "")
            search_url = f"https://www.dlsite.com/maniax/fsr/=/language/jp/sex_category%5B0%5D/male/keyword/{keyword}/order%5B0%5D/trend/options_and_or/and/per_page/30/from/fs.header"
            response = requests.get(search_url)
            soup = BeautifulSoup(response.content, "html.parser")
            product_urls = soup.find_all("a")

            # ページから作品のリンクを取得するための特定のタグやクラスを見つける
            product_urls = soup.find_all("a", href=True)

            # すべてのジャンル名を格納するリスト
            genres = set()

            for url in product_urls:
                product_link = url['href']
                if "product_id" in product_link:  # product_idが含まれるリンクだけを対象とする
                    product_response = requests.get(product_link)
                    product_soup = BeautifulSoup(product_response.content, "html.parser")

                    # /maniax/fsr/=/genre/*番号*/ のパターンを持つリンクを取得（class="btn_default"の有無に関わらず）
                    genre_links = product_soup.find_all("a", href=re.compile(r'/maniax/fsr/=/genre/\d+/'))

                    # 取得したリンクから<a>タグのテキスト内容を抽出し、genres集合に追加
                    for genre_link in genre_links:
                        genres.add(genre_link.text)

                    # <ul class="meny_selected_list">の中の<li class="many_selected_item">を探す
                    selected_items = product_soup.select("ul.meny_selected_list > li.many_selected_item")

                    # 各<li>タグ内の<a>タグのテキストをgenres集合に追加
                    for item in selected_items:
                        if item.a:  # <a>タグが存在する場合
                           genres.add(item.a.text)
                           
            # ジャンル名をカンマで区切って1つの文字列に連結
            if genres:
               genres_text = ", ".join(genres)

            else:
               genres_text = "ジャンルが見つかりませんでした"  # ジャンルが見つからなかった場合の文字列
                
            # フォルダ作成日時を取得
            created_date = datetime.fromtimestamp(os.path.getctime(subdir_path)).strftime("%Y/%m/%d")

            # Excelファイルに書き込み
            ws.append([category, author, title, subdir_path, created_date, genres_text])
            
            # ハイパーリンクを設定
            hyperlink = f'=HYPERLINK("{subdir_path}\\", "{title}")'
            cell = ws.cell(row=ws.max_row, column=4)
            cell.value = None
            cell.hyperlink = hyperlink
            cell.font = Font(color="0563C1", underline="single")
            cell.alignment = Alignment(horizontal="center", vertical="center")
            cell.hyperlink = None  # ここでcell.hyperlinkを削除

# Excelファイルを保存
wb.save(os.path.join(dir_path, filename))

# キーを一つ押してからスクリプトを終了する
input("Press Enter to exit...")

